Model

Trait Model 

pub trait Model:
    Serialize
    + DeserializeOwned
    + Send
    + Sync
    + Clone {
Show 38 methods // Required methods fn table_name() -> &'static str; fn get_primary_key(&self) -> Option<i64>; fn set_primary_key(&mut self, id: i64); fn columns() -> Vec<&'static str>; fn migration_sql() -> String; fn to_map(&self) -> Result<HashMap<String, Value>>; fn from_map(map: HashMap<String, Value>) -> Result<Self>; // Provided methods fn primary_key() -> &'static str { ... } async fn create(&self, db: &Database) -> Result<Self> { ... } async fn create_or_update(&self, db: &Database) -> Result<Self> { ... } async fn upsert( &self, unique_columns: &[&str], db: &Database, ) -> Result<Self> { ... } async fn bulk_create(models: &[Self], db: &Database) -> Result<Vec<Self>> { ... } async fn find_by_id(id: i64, db: &Database) -> Result<Option<Self>> { ... } async fn find_one( filter: FilterOperator, db: &Database, ) -> Result<Option<Self>> { ... } async fn find_all(db: &Database) -> Result<Vec<Self>> { ... } async fn find_where( filter: FilterOperator, db: &Database, ) -> Result<Vec<Self>> { ... } async fn find_paginated( pagination: &Pagination, db: &Database, ) -> Result<PaginatedResult<Self>> { ... } async fn find_where_paginated( filter: FilterOperator, pagination: &Pagination, db: &Database, ) -> Result<PaginatedResult<Self>> { ... } async fn search( search_filter: &SearchFilter, pagination: Option<&Pagination>, db: &Database, ) -> Result<PaginatedResult<Self>> { ... } async fn count(db: &Database) -> Result<u64> { ... } async fn count_where(filter: FilterOperator, db: &Database) -> Result<u64> { ... } async fn update(&self, db: &Database) -> Result<Self> { ... } async fn bulk_update(models: &[Self], db: &Database) -> Result<Vec<Self>> { ... } async fn delete(&self, db: &Database) -> Result<bool> { ... } async fn bulk_delete(ids: &[i64], db: &Database) -> Result<u64> { ... } async fn delete_where(filter: FilterOperator, db: &Database) -> Result<u64> { ... } async fn list( sort: Option<Vec<Sort>>, pagination: Option<&Pagination>, db: &Database, ) -> Result<PaginatedResult<Self>> { ... } async fn list_where( filter: FilterOperator, sort: Option<Vec<Sort>>, pagination: Option<&Pagination>, db: &Database, ) -> Result<PaginatedResult<Self>> { ... } async fn query(builder: QueryBuilder, db: &Database) -> Result<Vec<Self>> { ... } async fn query_paginated( builder: QueryBuilder, pagination: &Pagination, db: &Database, ) -> Result<PaginatedResult<Self>> { ... } async fn aggregate( function: Aggregate, column: &str, filter: Option<FilterOperator>, db: &Database, ) -> Result<Option<f64>> { ... } fn row_to_map(row: &LibsqlRow) -> Result<HashMap<String, Value>> { ... } fn value_to_libsql_value(value: &Value) -> LibsqlValue { ... } fn libsql_value_to_value(value: &LibsqlValue) -> Value { ... } fn log_info(message: &str) { ... } fn log_debug(message: &str) { ... } fn log_warn(message: &str) { ... } fn log_error(message: &str) { ... }
}
Core trait for all database models

Required Methods

fn table_name() -> &'static str

Get the table name for this model

fn get_primary_key(&self) -> Option<i64>

Get the primary key value

fn set_primary_key(&mut self, id: i64)

Set the primary key value

fn columns() -> Vec<&'static str>

Get all column names for this model

fn migration_sql() -> String

Generate SQL for creating the table

fn to_map(&self) -> Result<HashMap<String, Value>>

Convert the model to a HashMap for database operations

fn from_map(map: HashMap<String, Value>) -> Result<Self>

Create a model from a HashMap

Provided Methods

fn primary_key() -> &'static str

Get the primary key column name

async fn create(&self, db: &Database) -> Result<Self>

Create a new record in the database

async fn create_or_update(&self, db: &Database) -> Result<Self>

Create or update a record based on whether it has a primary key

async fn upsert(&self, unique_columns: &[&str], db: &Database) -> Result<Self>

Create or update a record based on unique constraints

async fn bulk_create(models: &[Self], db: &Database) -> Result<Vec<Self>>

Create multiple records in the database

async fn find_by_id(id: i64, db: &Database) -> Result<Option<Self>>

Find a record by its primary key

async fn find_one(filter: FilterOperator, db: &Database) -> Result<Option<Self>>

Find a single record by a specific condition

async fn find_all(db: &Database) -> Result<Vec<Self>>

Find all records

async fn find_where(filter: FilterOperator, db: &Database) -> Result<Vec<Self>>

Find records with a filter

async fn find_paginated( pagination: &Pagination, db: &Database, ) -> Result<PaginatedResult<Self>>

Find records with pagination

async fn find_where_paginated( filter: FilterOperator, pagination: &Pagination, db: &Database, ) -> Result<PaginatedResult<Self>>

Find records with filter and pagination

async fn search( search_filter: &SearchFilter, pagination: Option<&Pagination>, db: &Database, ) -> Result<PaginatedResult<Self>>

Search records with text search

async fn count(db: &Database) -> Result<u64>

Count all records

async fn count_where(filter: FilterOperator, db: &Database) -> Result<u64>

Count records with a filter

async fn update(&self, db: &Database) -> Result<Self>

Update a record

async fn bulk_update(models: &[Self], db: &Database) -> Result<Vec<Self>>

Update multiple records

async fn delete(&self, db: &Database) -> Result<bool>

Delete a record

async fn bulk_delete(ids: &[i64], db: &Database) -> Result<u64>

Delete multiple records

async fn delete_where(filter: FilterOperator, db: &Database) -> Result<u64>

Delete records with a filter

async fn list( sort: Option<Vec<Sort>>, pagination: Option<&Pagination>, db: &Database, ) -> Result<PaginatedResult<Self>>

List records with optional sorting and pagination

async fn list_where( filter: FilterOperator, sort: Option<Vec<Sort>>, pagination: Option<&Pagination>, db: &Database, ) -> Result<PaginatedResult<Self>>

List records with filter, sorting, and pagination

async fn query(builder: QueryBuilder, db: &Database) -> Result<Vec<Self>>

Execute a custom query

async fn query_paginated( builder: QueryBuilder, pagination: &Pagination, db: &Database, ) -> Result<PaginatedResult<Self>>

Execute a custom query with pagination

async fn aggregate( function: Aggregate, column: &str, filter: Option<FilterOperator>, db: &Database, ) -> Result<Option<f64>>

Get aggregate value

fn row_to_map(row: &LibsqlRow) -> Result<HashMap<String, Value>>

Convert a database row to a HashMap

fn value_to_libsql_value(value: &Value) -> LibsqlValue

Convert our Value type to crate::compat::LibsqlValue

fn libsql_value_to_value(value: &LibsqlValue) -> Value

Convert crate::compat::LibsqlValue to our Value type

fn log_info(message: &str)

Log an info message

fn log_debug(message: &str)

Log a debug message

fn log_warn(message: &str)

Log a warning message

fn log_error(message: &str)

Log an error message

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors